@@ -15,7 +15,6 @@
 from collections import Counter
 from collections import OrderedDict
 from collections import abc
-from collections import defaultdict
 from copy import copy
 from glob import glob
 from itertools import chain
@@ -25,7 +24,6 @@ from logging import getLogger
 from typing import Dict
 from typing import Iterable
 from typing import List
-from typing import Mapping
 from typing import Tuple
 import hashlib
 import importlib.util
@@ -35,7 +33,6 @@ import sys
 import inspect
 import types
 import textwrap
-import weakref
 
 import pkg_resources
 import sqlparse
@@ -48,10 +45,6 @@ default_migration_table = "_yoyo_migration"
 
 hash_function = hashlib.sha256
 
-_collectors = (
-    weakref.WeakValueDictionary()
-)  # type: Mapping[str, "StepCollector"]
-
 
 def _is_migration_file(path):
     """
@@ -92,9 +85,7 @@ def parse_metadata_from_sql_comments(
     directive_names = ["transactional", "depends"]
     comment_or_empty = re.compile(r"^(\s*|\s*--.*)$").match
     directive_pattern = re.compile(
-        r"^\s*--\s*({})\s*:\s*(.*)$".format(
-            "|".join(map(re.escape, directive_names))
-        )
+        r"^\s*--\s*({})\s*:\s*(.*)$".format("|".join(map(re.escape, directive_names)))
     )
 
     lineending = re.search(r"\n|\r\n|\r", s + "\n").group(0)  # type: ignore
@@ -161,9 +152,7 @@ class Migration(object):
         self.module = None
 
     def __repr__(self):
-        return "<{} {!r} from {}>".format(
-            self.__class__.__name__, self.id, self.path
-        )
+        return "<{} {!r} from {}>".format(self.__class__.__name__, self.id, self.path)
 
     def is_raw_sql(self):
         return self.path.endswith(".sql")
@@ -182,7 +171,6 @@ class Migration(object):
             return
 
         collector = StepCollector(migration=self)
-        _collectors[self.path] = collector
         with open(self.path, "r") as f:
             self.source = f.read()
 
@@ -190,16 +178,15 @@ class Migration(object):
             self.module = types.ModuleType(self.path)
         else:
             spec = importlib.util.spec_from_file_location(self.path, self.path)
+            assert spec is not None
             self.module = importlib.util.module_from_spec(spec)
 
         self.module.step = collector.add_step  # type: ignore
         self.module.group = collector.add_step_group  # type: ignore
         self.module.transaction = collector.add_step_group  # type: ignore
-        self.module.collector = collector  # type: ignore
+        self.module.__yoyo_collector__ = collector  # type: ignore
         if self.is_raw_sql():
-            directives, leading_comment, statements = read_sql_migration(
-                self.path
-            )
+            directives, leading_comment, statements = read_sql_migration(self.path)
             _, _, rollback_statements = read_sql_migration(
                 os.path.splitext(self.path)[0] + ".rollback.sql"
             )
@@ -209,23 +196,35 @@ class Migration(object):
             )
 
             for s, r in statements_with_rollback:
-                self.module.collector.add_step(s, r)  # type: ignore
+                collector.add_step(s, r)
             self.module.__doc__ = leading_comment
-            self.module.__transactional__ = {"true": True, "false": False}[  # type: ignore
-                directives.get("transactional", "true").lower()
-            ]
-            self.module.__depends__ = {  # type: ignore
-                d for d in directives.get("depends", "").split() if d
-            }
+            setattr(
+                self.module,
+                "__transactional__",
+                {"true": True, "false": False}[
+                    directives.get("transactional", "true").lower()
+                ],
+            )
+            setattr(
+                self.module,
+                "__depends__",
+                {d for d in directives.get("depends", "").split() if d},
+            )
 
         else:
             try:
-                spec.loader.exec_module(self.module)
+                if spec and spec.loader:
+                    spec.loader.exec_module(self.module)
+                else:
+                    logger.exception(
+                        "Could not import migration from %r: "
+                        "ModuleSpec has no loader attached",
+                        self.path,
+                    )
+                    raise exceptions.BadMigration(self.path)
 
             except Exception as e:
-                logger.exception(
-                    "Could not import migration from %r: %r", self.path, e
-                )
+                logger.exception("Could not import migration from %r: %r", self.path, e)
                 raise exceptions.BadMigration(self.path, e)
         depends = getattr(self.module, "__depends__", [])
         if isinstance(depends, (str, bytes)):
@@ -244,6 +243,7 @@ class Migration(object):
         reverse = {"rollback": "apply", "apply": "rollback"}[direction]
 
         steps = self.steps
+        assert steps is not None
         if direction == "rollback":
             steps = reversed(steps)  # type: ignore
 
@@ -261,10 +261,7 @@ class Migration(object):
                 except backend.DatabaseError:
                     exc_info = sys.exc_info()
 
-                    if (
-                        not backend.has_transactional_ddl
-                        or not self.use_transactions
-                    ):
+                    if not backend.has_transactional_ddl or not self.use_transactions:
                         # Any DDL statements that have been executed have been
                         # committed. Go through the rollback steps to undo
                         # these inasmuch is possible.
@@ -272,10 +269,8 @@ class Migration(object):
                             for step in reversed(executed_steps):
                                 getattr(step, reverse)(backend)
                         except backend.DatabaseError:
-                            logger.exception(
-                                "Could not %s step %s", direction, step.id
-                            )
-                    if exc_info[1]:
+                            logger.exception("Could not %s step %s", direction, step.id)
+                    if exc_info[1] is not None:
                         raise exc_info[1].with_traceback(exc_info[2])
 
 
@@ -387,9 +382,7 @@ class MigrationStep(StepBase):
             logger.debug(" - executing %r", stmt)
         cursor.execute(stmt)
         if cursor.description:
-            result = [
-                [str(value) for value in row] for row in cursor.fetchall()
-            ]
+            result = [[str(value) for value in row] for row in cursor.fetchall()]
             column_names = [desc[0] for desc in cursor.description]
             column_sizes = [len(c) for c in column_names]
 
@@ -400,9 +393,7 @@ class MigrationStep(StepBase):
             format = "|".join(" %%- %ds " % size for size in column_sizes)
             format += "\n"
             out.write(format % tuple(column_names))
-            out.write(
-                "+".join("-" * (size + 2) for size in column_sizes) + "\n"
-            )
+            out.write("+".join("-" * (size + 2) for size in column_sizes) + "\n")
             for row in result:
                 out.write(format % tuple(row))
             out.write(plural(len(result), "(%d row)", "(%d rows)") + "\n")
@@ -483,10 +474,10 @@ def _expand_sources(sources) -> Iterable[Tuple[str, List[str]]]:
             for directory in glob(source):
                 paths = [
                     os.path.join(directory, path)
-                    for path in sorted(os.listdir(directory))
+                    for path in os.listdir(directory)
                     if _is_migration_file(path)
                 ]
-                yield (directory, paths)
+                yield (directory, sorted(paths))
 
 
 def read_migrations(*sources):
@@ -584,9 +575,7 @@ class MigrationList(abc.MutableSequence):
         return ob
 
     def filter(self, predicate):
-        return self.__class__(
-            [m for m in self if predicate(m)], self.post_apply
-        )
+        return self.__class__([m for m in self if predicate(m)], self.post_apply)
 
     def replace(self, newmigrations):
         return self.__class__(newmigrations, self.post_apply)
@@ -613,12 +602,8 @@ class StepCollector(object):
         """
 
         def do_add(use_transactions):
-            wrapper = (
-                TransactionWrapper if use_transactions else Transactionless
-            )
-            t = MigrationStep(
-                next(self.step_id), apply, rollback
-            )  # type: StepBase
+            wrapper = TransactionWrapper if use_transactions else Transactionless
+            t = MigrationStep(next(self.step_id), apply, rollback)  # type: StepBase
             t = wrapper(t, ignore_errors)
             return t
 
@@ -639,22 +624,16 @@ class StepCollector(object):
             steps = kwargs["steps"]
         else:
             steps = list(
-                chain(
-                    *(s if isinstance(s, abc.Iterable) else [s] for s in args)
-                )
+                chain(*(s if isinstance(s, abc.Iterable) else [s] for s in args))
             )
         for s in steps:
             del self.steps[s]
 
         def do_add(use_transactions):
             ignore_errors = kwargs.pop("ignore_errors", None)
-            wrapper = (
-                TransactionWrapper if use_transactions else Transactionless
-            )
+            wrapper = TransactionWrapper if use_transactions else Transactionless
 
-            group = StepGroup(
-                [create_step(use_transactions) for create_step in steps]
-            )
+            group = StepGroup([create_step(use_transactions) for create_step in steps])
             return wrapper(group, ignore_errors)
 
         self.steps[do_add] = 1
@@ -664,14 +643,17 @@ class StepCollector(object):
         return [create_step(use_transactions) for create_step in self.steps]
 
 
-def _get_collector(depth=2):
-    for stackframe in reversed(inspect.stack()):
-        path = stackframe.frame.f_code.co_filename
-        if path in _collectors:
-            return _collectors[path]
-    raise AssertionError(
-        "Excected to be called in the context of a migration module import"
-    )
+def _get_collector():
+    frame = inspect.currentframe()
+    try:
+        while frame is not None:
+            if "__yoyo_collector__" in frame.f_globals:
+                return frame.f_globals["__yoyo_collector__"]
+            frame = frame.f_back
+    except KeyError:
+        raise AssertionError(
+            "Expected to be called in the context of a migration module import"
+        )
 
 
 def step(*args, **kwargs):
@@ -741,72 +723,18 @@ def heads(migration_list):
     return heads
 
 
-def topological_sort(migration_list: MigrationList) -> Iterable[Migration]:
+def topological_sort(migrations: Iterable[Migration]) -> Iterable[Migration]:
+    from yoyo.topologicalsort import topological_sort as topological_sort_impl
+    from yoyo.topologicalsort import CycleError
 
-    # Make a copy of migration_list. It's probably an iterator.
-    migration_list = list(migration_list)
-
-    # Track graph edges in two parallel data structures.
-    # Use OrderedDict so that we can traverse edges in order
-    # and keep the sort stable
-    forward_edges = defaultdict(
-        OrderedDict
-    )  # type: Dict[Migration, Dict[Migration, int]]
-    backward_edges = defaultdict(
-        OrderedDict
-    )  # type: Dict[Migration, Dict[Migration, int]]
-
-    def sort_by_stability_order(
-        items, ordering={m: index for index, m in enumerate(migration_list)}
-    ):
-        return sorted(
-            (item for item in items if item in ordering), key=ordering.get
+    migration_list = list(migrations)
+    all_migrations = set(migration_list)
+    dependency_graph = {m: (m.depends & all_migrations) for m in migration_list}
+    try:
+        return topological_sort_impl(migration_list, dependency_graph)
+    except CycleError as e:
+        raise exceptions.BadMigration(
+            "Circular dependencies among these migrations {}".format(
+                ", ".join(m.id for m in e.args[1])
+            )
         )
-
-    for m in migration_list:
-        for n in sort_by_stability_order(m.depends):
-            forward_edges[n][m] = 1
-            backward_edges[m][n] = 1
-
-    def check_cycles(item):
-        stack = [item]
-        path = []
-        while stack:
-            n = stack.pop()
-            if n in path:
-                raise exceptions.BadMigration(
-                    "Circular dependencies among these migrations {}".format(
-                        ", ".join(m.id for m in path + [n])
-                    )
-                )
-            path.append(n)
-            stack.extend(forward_edges[n])
-
-    seen = set()
-    for item in migration_list:
-
-        if item in seen:
-            continue
-
-        check_cycles(item)
-
-        # if item is in a dedepency graph, go back to the root node
-        while backward_edges[item]:
-            item = next(iter(backward_edges[item]))
-
-        # is item at the start of a dependency graph?
-        if forward_edges[item]:
-            stack = [item]
-            while stack:
-                m = stack.pop()
-                yield m
-                seen.add(m)
-                for child in list(reversed(forward_edges[m])):
-                    if all(
-                        dependency in seen
-                        for dependency in backward_edges[child]
-                    ):
-                        stack.append(child)
-        else:
-            yield item
-            seen.add(item)
