@@ -70,14 +70,12 @@ def install_argparsers(global_parser, subparsers):
     filter_parser.add_argument(
         "-r",
         "--revision",
-        help="Apply/rollback migration with id " "REVISION",
+        help=("Apply/rollback migration with id REVISION and all its dependencies"),
         metavar="REVISION",
     )
 
     # Options related to applying/rolling back migrations
-    apply_parser = argparse.ArgumentParser(
-        add_help=False, parents=[filter_parser]
-    )
+    apply_parser = argparse.ArgumentParser(add_help=False, parents=[filter_parser])
     apply_parser.add_argument(
         "-a",
         "--all",
@@ -91,20 +89,39 @@ def install_argparsers(global_parser, subparsers):
         "--force",
         dest="force",
         action="store_true",
-        help="Force apply/rollback of steps even if "
-        "previous steps have failed",
+        help="Force apply/rollback of steps even if previous steps have failed",
     )
 
     parser_apply = subparsers.add_parser(
         "apply",
-        description="Apply migrations",
+        help="Apply migrations",
         parents=[global_parser, standard_options_parser, apply_parser],
     )
     parser_apply.set_defaults(func=apply, command_name="apply")
+    parser_apply.add_argument(
+        "-1",
+        "--one",
+        help="Apply a single migration. "
+        "If there are no unapplied migrations, reapply the last migration",
+    )
+
+    parser_develop = subparsers.add_parser(
+        "develop",
+        help="Apply migrations without prompting. "
+        "If there are no unapplied migrations, reapply the last migration",
+        parents=[global_parser, standard_options_parser, apply_parser],
+    )
+    parser_develop.set_defaults(func=develop, command_name="develop")
+    parser_develop.add_argument(
+        "-n",
+        type=int,
+        help="Act on the last N migrations",
+        default=1,
+    )
 
     parser_list = subparsers.add_parser(
         "list",
-        description="List all available and applied migrations. "
+        help="List all available and applied migrations. "
         "Each listed migration is prefixed with either A (applied) or U "
         "(unapplied)",
         parents=[global_parser, standard_options_parser, filter_parser],
@@ -114,14 +131,14 @@ def install_argparsers(global_parser, subparsers):
     parser_rollback = subparsers.add_parser(
         "rollback",
         parents=[global_parser, standard_options_parser, apply_parser],
-        description="Rollback migrations",
+        help="Rollback migrations",
     )
     parser_rollback.set_defaults(func=rollback, command_name="rollback")
 
     parser_reapply = subparsers.add_parser(
         "reapply",
         parents=[global_parser, standard_options_parser, apply_parser],
-        description="Reapply migrations",
+        help="Rollback then reapply migrations",
     )
     parser_reapply.set_defaults(func=reapply, command_name="reapply")
 
@@ -135,12 +152,14 @@ def install_argparsers(global_parser, subparsers):
     parser_unmark = subparsers.add_parser(
         "unmark",
         parents=[global_parser, standard_options_parser, apply_parser],
-        description="Unmark applied migrations, without rolling them back",
+        help="Unmark applied migrations, without rolling them back",
     )
     parser_unmark.set_defaults(func=unmark, command_name="unmark")
 
     parser_break_lock = subparsers.add_parser(
-        "break-lock", parents=[global_parser, standard_options_parser], help="Break migration locks"
+        "break-lock",
+        parents=[global_parser, standard_options_parser],
+        help="Break migration locks",
     )
     parser_break_lock.set_defaults(func=break_lock, command_name="break-lock")
 
@@ -161,9 +180,7 @@ def migrations_to_revision(migrations, revision, direction):
 
     targets = [m for m in migrations if revision in m.id]
     if len(targets) == 0:
-        raise InvalidArgument(
-            "'{}' doesn't match any revisions.".format(revision)
-        )
+        raise InvalidArgument("'{}' doesn't match any revisions.".format(revision))
     if len(targets) > 1:
         raise InvalidArgument(
             "'{}' matches multiple revisions. "
@@ -174,7 +191,7 @@ def migrations_to_revision(migrations, revision, direction):
 
     target = targets[0]
 
-    # apply: apply target an all its dependencies
+    # apply: apply target and all its dependencies
     if direction == "apply":
         deps = ancestors(target, migrations)
         target_plus_deps = deps | {target}
@@ -189,7 +206,7 @@ def migrations_to_revision(migrations, revision, direction):
     return migrations
 
 
-def get_migrations(args, backend):
+def get_migrations(args, backend, direction=None):
 
     sources = args.sources
     dburi = args.database
@@ -197,7 +214,8 @@ def get_migrations(args, backend):
     if not sources:
         raise InvalidArgument("Please specify the migration source directory")
 
-    direction = "apply" if args.func in {mark, apply} else "rollback"
+    if not direction:
+        direction = "apply" if args.func in {mark, apply} else "rollback"
     migrations = read_migrations(*sources)
     migrations = filter_migrations(migrations, args.match)
     migrations = migrations_to_revision(migrations, args.revision, direction)
@@ -210,12 +228,7 @@ def get_migrations(args, backend):
     if not args.batch_mode and not args.revision:
         migrations = prompt_migrations(backend, migrations, args.command_name)
 
-    if (
-        args.batch_mode
-        and not args.revision
-        and not args.all
-        and args.func is rollback
-    ):
+    if args.batch_mode and not args.revision and not args.all and args.func is rollback:
         if len(migrations) > 1:
             warnings.warn(
                 "Only rolling back a single migration."
@@ -235,9 +248,7 @@ def get_migrations(args, backend):
             print("  [{m.id}]".format(m=m))
         prompt = "{} {} to {}".format(
             args.command_name.title(),
-            utils.plural(
-                len(migrations), "this migration", "these %d migrations"
-            ),
+            utils.plural(len(migrations), "this migration", "these %d migrations"),
             dburi,
         )
         if not utils.confirm(prompt, default="y"):
@@ -268,6 +279,32 @@ def rollback(args, config):
         backend.rollback_migrations(migrations, args.force)
 
 
+def develop(args, config):
+    args.batch_mode = True
+    backend = get_backend(args, config)
+    with backend.lock():
+        migrations = get_migrations(args, backend, "apply")
+        if migrations:
+            print(
+                "Applying",
+                utils.plural(len(migrations), "%d migration:", "%d migrations:"),
+            )
+            for m in migrations:
+                print(f"  [{m.id}]")
+            backend.apply_migrations(migrations, args.force)
+        else:
+            migrations = get_migrations(args, backend, "rollback")[: args.n]
+            print(
+                "Reapplying",
+                utils.plural(len(migrations), "%d migration:", "%d migrations:"),
+            )
+            for m in migrations:
+                print(f"  [{m.id}]")
+            backend.rollback_migrations(migrations, args.force)
+            migrations = get_migrations(args, backend, "apply")
+            backend.apply_migrations(migrations, args.force)
+
+
 def mark(args, config):
     backend = get_backend(args, config)
     with backend.lock():
@@ -336,15 +373,11 @@ def prompt_migrations(backend, migrations, direction):
                 choice = "n" if is_applied else "y"
             else:
                 choice = "y" if is_applied else "n"
-        options = "".join(
-            o.upper() if o == choice else o.lower() for o in "ynvdaqjk?"
-        )
+        options = "".join(o.upper() if o == choice else o.lower() for o in "ynvdaqjk?")
 
         print("")
         print("[%s]" % (mig.migration.id,))
-        response = utils.prompt(
-            "Shall I %s this migration?" % (direction,), options
-        )
+        response = utils.prompt("Shall I %s this migration?" % (direction,), options)
 
         if response == "?":
             print("")
@@ -354,8 +387,7 @@ def prompt_migrations(backend, migrations, direction):
             print("v: view this migration in full")
             print("")
             print(
-                "d: %s the selected migrations, skipping any remaining"
-                % (direction,)
+                "d: %s the selected migrations, skipping any remaining" % (direction,)
             )
             print("a: %s all the remaining migrations" % (direction,))
             print("q: cancel without making any changes")
